﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using HtmlAgilityPack;
using Newtonsoft.Json.Linq;
using Quartz;
using SCC.Common;
using SCC.Crawler.Tools;
using SCC.Interface;
using SCC.Models;

namespace SCC.Crawler.DT
{
    /// <summary>
    ///     数据爬取类
    ///     华东15选5
    /// </summary>
    [DisallowConcurrentExecution]
    [PersistJobDataAfterExecution]
    public class HD15X5Job : IJob
    {
        /// <summary>
        ///     构造函数
        /// </summary>
        public HD15X5Job()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            email = IOC.Resolve<IEmail>();
        }

        /// <summary>
        ///     作业执行入口
        /// </summary>
        /// <param name="context">作业执行上下文</param>
        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);


         
                var OpenList = GetOpenListFromMainUrl(Config.MainUrl);
                foreach (var openList in OpenList)
                {
                    LatestItem = openList;
                    if (LatestItem != null && services.AddDTOpen5Code(currentLottery, LatestItem))
                    {
                        log.Info(GetType(), CommonHelper.GetJobMainLogInfo(Config, openList.Term.ToString()));
                        isGetData = true;
                    }
                }
                //}


                ////保存最新期号
                context.JobDetail.JobDataMap["LatestItem"] = LatestItem;
            

        }
        /// <summary>
        /// 根据url获取数据列表
        /// </summary>
        /// <param name="mainUrl"></param>
        /// <returns></returns>
        private List<OpenCode5DTModel> GetOpenListFromMainUrl(string mainUrl)
        {
            var result = new List<OpenCode5DTModel>();
            try
            {
                var postData = "PlayType=5&currentPage=1&pageSize=500";
                var htmlResource = NetHelper.GetUrlResponse(mainUrl, "POST", postData, Encoding.UTF8);

                if (htmlResource == null) return result;


                var obj = htmlResource.JsonToEntity<JObject>();
                var lotteryList = obj["LotteryNumberList"];
                OpenCode5DTModel model = null;
                string[] openCodeList = null;
                var issue = string.Empty;
                foreach (var item in lotteryList)
                {
                    issue = item["Issue"].ToString();
                 
                    model = new OpenCode5DTModel();
                    model.Term = Convert.ToInt64(issue);
                    model.OpenTime = Convert.ToDateTime(item["LotteryDate"].ToString());
                    openCodeList = item["BasicNumber"].ToString().Trim().Replace("[", string.Empty)
                        .Replace("]", string.Empty).Replace("\r\n", string.Empty).Replace("\"", string.Empty)
                        .Split(',');
                    if (openCodeList.Length != 5) continue;
                    model.OpenCode1 = Convert.ToInt32(openCodeList[0]);
                    model.OpenCode2 = Convert.ToInt32(openCodeList[1]);
                    model.OpenCode3 = Convert.ToInt32(openCodeList[2]);
                    model.OpenCode4 = Convert.ToInt32(openCodeList[3]);
                    model.OpenCode5 = Convert.ToInt32(openCodeList[4]);
                    model.Spare = GetKaijiangDetails(issue);
                    result.Add(model);
                }

             
            }
            catch (Exception ex)
            {
                log.Error(GetType(),
                    string.Format("【{0}】通过主站点抓取开奖列表时发生错误，错误信息【{1}】", Config.Area + currentLottery, ex.Message));
            }
            return result;
        }

        private string GetKaijiangDetails(string Term)
        {

            string Url = "https://www.cp2y.com/15x5/15x5kj/" + Term+"/";
            var htmlResource = NetHelper.GetUrlResponse(Url);
            if (htmlResource == null) return null;
            var doc = new HtmlDocument();
            doc.LoadHtml(htmlResource);
            var div = doc.DocumentNode.SelectSingleNode("//div[@class='history-content']");
            var table = doc.DocumentNode.SelectSingleNode("//table");
            var trs = table.ChildNodes.Where(node => node.Name == "tr").ToList();
            var   divs= doc.DocumentNode.SelectSingleNode("//div[@class='fl lh24 f14']");
            var  xiangqing=divs.ChildNodes.Where(node => node.Name == "div").ToList();
            var Trje = xiangqing[1].InnerText.Substring(26).Replace(",","").Replace("元", "").Trim();
            var Gdje = xiangqing[2].InnerText.Substring(24).Replace(",", "").Replace("元", "").Replace("--", "0").Trim();
            if (div == null) return null;

            var entity = new KaijiangDetailsEntity
            {
                Trje = Trje == "0" ? "0" : (double.Parse(Trje) * 1).ToString(),
                Gdje = Gdje == "0" ? "0" : (double.Parse(Gdje) * 1).ToString()
            };
            var list = new List<Kaijiangitem>();
            for (var i = 1; i < trs.Count; i++)
            {
                var tds = trs[i].ChildNodes.Where(node => node.Name == "td").ToList();


                var kaijiangitem = new Kaijiangitem();

                var TotalMoney = tds[3].InnerText.Replace("元", "").Replace("--", "0").Replace(",", "").Trim();
                kaijiangitem.Name = tds[1].InnerText.Trim();
                kaijiangitem.TotalMoney = TotalMoney == "0" ? "0" : double.Parse(TotalMoney).ToString();
                kaijiangitem.Total = tds[2].InnerText.Replace(" 注", "").Replace("--", "0").Trim();
                list.Add(kaijiangitem);
            }

            entity.KaiJiangItems = list;
            return entity.TryToJson();

        }
        #region Attribute

        /// <summary>
        ///     配置信息
        /// </summary>
        private SCCConfig Config;

        /// <summary>
        ///     当天抓取的最新一期开奖记录
        /// </summary>
        private OpenCode5DTModel LatestItem;

        /// <summary>
        ///     当天抓取失败列表
        /// </summary>
        private List<string> FailedQiHaoList = null;

        /// <summary>
        ///     日志对象
        /// </summary>
        private readonly LogHelper log;

        /// <summary>
        ///     数据服务
        /// </summary>
        private readonly IDTOpenCode services;

        /// <summary>
        ///     当前彩种
        /// </summary>
        private SCCLottery currentLottery => SCCLottery.HD15X5;

        /// <summary>
        ///     邮件接口
        /// </summary>
        private IEmail email;

        /// <summary>
        ///     是否本次运行抓取到开奖数据
        /// </summary>
        private bool isGetData;

        #endregion
    }
}